
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    require Test::More;
    Test::More::plan(skip_all => 'these tests are for testing by the author');
  }
}

use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::NoTabs 0.15

use Test::More 0.88;
use Test::NoTabs;

my @files = (
    'lib/WebService/PivotalTracker.pm',
    'lib/WebService/PivotalTracker/Client.pm',
    'lib/WebService/PivotalTracker/Comment.pm',
    'lib/WebService/PivotalTracker/Entity.pm',
    'lib/WebService/PivotalTracker/Label.pm',
    'lib/WebService/PivotalTracker/Me.pm',
    'lib/WebService/PivotalTracker/PropertyAttributes.pm',
    'lib/WebService/PivotalTracker/Story.pm',
    'lib/WebService/PivotalTracker/Types.pm',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/author-00-compile.t',
    't/author-eol.t',
    't/author-no-tabs.t',
    't/author-pod-spell.t',
    't/author-pod-syntax.t',
    't/author-test-version.t',
    't/basic.t',
    't/fixtures/services/v5/stories/120292647.json',
    't/lib/Test/UA.pm',
    't/release-cpan-changes.t',
    't/release-portability.t',
    't/release-synopsis.t',
    't/release-tidyall.t'
);

notabs_ok($_) foreach @files;
done_testing;
